/* Copyright (C) Seth A. Robinson, 2003. 
 * All rights reserved worldwide.
 *
 * This software is provided "as is" without express or implied
 * warranties. You may freely copy and compile this source into
 * applications you distribute provided that the copyright text
 * below is included in the resulting source code, for example:
 * "Portions Copyright (C) Seth A. Robinson, 2003"
 */

//Last updated 7-21-2003


//

//  This is a group of somewhat generic utilities that can be used with MSVC 6.  Can be compiled with
//  with or without MFC, ANSI or unicode.  Not all functions are unicode aware though.

//  Also, some functions use TCHAR and some use WCHAR, the reason is I needed some unicode functions
//  in non-unicode builds and vice-versa, for instance, Teenage Lawnmower is not built with _UNICODE but it reads
//  unicode scripts.

// It's a mix-mash of useful functions, mixed styles and mixed quality, most written by Seth, some
// snipped from various sources.  Some of these date back ten years and could use rewriting...

//After adding this to your project you may get errors about LogMsg and LogError not
//being defined.  You need to add this to your program yourself, this is so you can choose
//how to handle error messages.  (write to disk or show on the screen or whatever)

//you can define _NOLOGMSG if you really don't want to use them.


//You can cut and paste these into your program to get you going:
/*


void LogMsg(const char *lpFormat, ...)
{
	va_list Marker;
	char szBuf[4048];
	va_start(Marker, lpFormat);
	vsprintf(szBuf, lpFormat, Marker);
	va_end(Marker);
	char stTemp[4048];
	sprintf(stTemp, "%s\r\n", szBuf);
    OutputDebugString(stTemp);
}

void LogError(const char *lpFormat, ...)
{
	va_list Marker;
	char szBuf[4048];
	va_start(Marker, lpFormat);
	vsprintf(szBuf, lpFormat, Marker);
	va_end(Marker);
	char stTemp[4048];
	sprintf(stTemp, "Error: %s\r\n", szBuf);
	OutputDebugString(stTemp);
}
*/


#pragma once

#pragma warning (disable:4786)
#include <windows.h>
#include <cstdio>
#include <tchar.h>

#ifndef SAFE_RELEASE
#define SAFE_RELEASE(p)      { if(p) { (p)->Release(); (p)=NULL; } }
#endif
#define SAFE_FREE(p)      { if(p) { free(p); (p)=NULL; } }
#define SAFE_DELETE_ARRAY(p) { if(p) { delete[] (p);   (p)=NULL; } }
#ifndef SAFE_DELETE
#define SAFE_DELETE(p)      { if(p) { delete (p); (p)=NULL; } }
#endif

#define GET_BYTE(l, bytenum)  (((l) >> (bytenum * 8)) & 0xFF)
#define RELEASE(x) if(x != NULL) { x->Release(); x = NULL; }

void LogMsg(const TCHAR * lpFormat, ...);
void LogError(const TCHAR * lpFormat, ...);
void switch_to_current_dir();
bool get_line_from_file(FILE * fp, char * dest);
char *replace(char *szSearch, char *szReplace, char *szBuffer);
int random( long i_max);
bool number_is_close(int i_orig, int i_target, int i_range);




