// Example showing how to use CWordList.  I used a console project for simplicity,
// but you can do this under Win32/MFC builds too.

// An "optimized" dictionary text file is included with this zip for this
// test (the one from Dungeon Scroll) it's limited to just 3 to 8 letter
// words.

// Optimized means it was loaded and saved as optimized, those are function in CWordList.  I did
// include a nice utility to do it with Dungeon Scroll if you want to use that, or you can easily
// write your own front end.

// When running, you probably don't want to give it more than 15 or so letters
// to make words with or you may be waiting quite a while...

// Special thanks to Alan Beale for his work on the Enhanced North American Benchmark LExicon (ENABLE).



#include <cstdlib>
#include <iostream>

using namespace std;

#include "All.h"
#include "CWordList.h"

/*
 All my classes report important messages or errors with these printf style functions, they must be
 defined in the project, so you can choose how to handle them. Here I'm just throwing them
 to OutputDebugString, but in a game it would probably be a console or something.

 You could also just do:

 void LogMsg(char const *,...) {};
 void LogError(char const *,...) {};

 if you don't care about the messages.
 
*/

void LogMsg(const char *lpFormat, ...)
{
	va_list Marker;
	char szBuf[4048];
	va_start(Marker, lpFormat);
	vsprintf(szBuf, lpFormat, Marker);
	va_end(Marker);
	char stTemp[4048];
	sprintf(stTemp, "%s\r\n", szBuf);
    OutputDebugString(stTemp);
}

void LogError(const char *lpFormat, ...)
{
	va_list Marker;
	char szBuf[4048];
	va_start(Marker, lpFormat);
	vsprintf(szBuf, lpFormat, Marker);
	va_end(Marker);
	char stTemp[4048];
	sprintf(stTemp, "Error: %s\r\n", szBuf);
	OutputDebugString(stTemp);
}

void WaitForKey()
{
  cout << "\nPlease press [RETURN] to continue: " << flush;
  cin.clear();
  cin.ignore(cin.rdbuf()->in_avail() , '\n');

  cin.get();
  cout << endl;
}

int main()
{
 //move to the directory that the .exe is in
 switch_to_current_dir();


 cout << "CWordList demo app.  Let's load the word list.\n\n";
 
 CWordList words;
 
 if (!words.ReadInOptimizedWordlist("master_word_list.txt"))
 {
     cout << "Oh my god, can't find the word list to load!\n";
#ifdef _DEBUG
     WaitForKey(); //retail mode has it's own pause automatically in MSVC 6/7
#endif
     return 0;
 }

 cout << "Loaded " << words.GetCountOfWordsLoaded() << " words.\n\n";


 //get entry from the user
 cout << "Ok, now enter a word.  I'll tell you if it's valid and also how many other words you can make with those"\
     "letters.\n\nEnter word or letters: ";
 
 string str_users_input;
 cin >> str_users_input;
 
 if (words.IsWordInDictionary(const_cast<char *>(str_users_input.c_str())))
 {
     //valid word
     cout << "\n\nGreat, '" << str_users_input << "' IS in the dictionary.  Checking for anagrams...";
 } else
 {
     //word not found in the dictionary
     cout << "\n\nUm, '" << str_users_input << "' isn't in the dictionary.  Checking for anagrams...";
 }

 cout << "\n\nWorking...";

 words.GenerateAnagrams(const_cast<char *>(str_users_input.c_str()), NULL);

 cout << "Found " << words.GetCountOfAnagramsFound() << " words that can be made with the letter set '" << str_users_input <<
     "'.\n\n";
 
 //should we list the words we found?
 if (words.GetCountOfAnagramsFound() > 0)
 {
      WaitForKey();
     
     //list all anagrams we found
     for (int i=0; i < words.GetCountOfAnagramsFound(); i++)
     {
         cout << "Word " << i+1 << ": '" << words.GetAnagramsFound()->GetWord(i) << "'\n";
     }
 }
#ifdef _DEBUG
 WaitForKey();
 cout << "\nShutting down.  This is really slow in debug mode...";
#endif

 return 0;
}